function [runlength, varargout] = RobustDFEWMA(minwin, maxwin, dim, kmax, m0, tau, delta, alpha, nbound, lambda, rawSample)
% This function simulate the run length of the nonparametric MSPC chart 
% win: is the window size to be used in the statistic
% dim: is the dimension of the variable
% kmax: the maximum number in the simulation
% m0: in control sample points
% tau: change point
% delta: mean shift size
% alpha: alpha-error of the conditional test
% nbound: the number of permutation required
% @dist: random variable generation for the in control data

% compute the mean and sd of the rank statistics
win = 1:kmax;
win = max(minwin, win);
win = min(win, maxwin);

sampleNum = m0 + (1:kmax);
mu = win.*(sampleNum + 1)/2;
sd = sqrt((win .* (sampleNum+1) .* (sampleNum-win))/12);

% initialize some variables in the code
climit = zeros(1, kmax); % control limit at each step
[Tval] = deal(climit); % rank statistic, and standardized version
runlength = kmax;

% allocate and initial the rank matrix
rankMat = zeros(kmax+m0, dim);

% sort and stored the rank in the matrix
for i = 1 : dim
    [~,idx] = sort(rawSample(1:m0,i));
    rankMat(idx, i) = 1:m0;
end

% generate sample from the distribution, and start runing MSPC
for i = 1 : kmax
%     tic;
    cidx = m0 + i;
    csample = rawSample(cidx,:); % generate a new sample
    if i > tau 
        csample = csample + delta; % mean shift
    end

    % update the rank of the whole samples
    eidx = 1:(cidx-1);
    for j = 1 : dim
        largerIdx = find(rawSample(eidx,j) > csample(j));
        rankMat(largerIdx,j) = rankMat(largerIdx, j) + 1; % increase larger sample by 1
        rankMat(cidx,j) = cidx - length(largerIdx); % rank of current sample
    end
    rawSample(cidx, :) = csample;

    % compute the rank statistics
    weight = (win(i)-1):(-1):0;
    weight = (1-lambda).^weight';
    wm = repmat(weight, 1, dim);
    Tval(i) = sum((sum(wm.*(rankMat((cidx-win(i)+1):cidx, :)-(cidx+1)/2), 1)).^2)/(sd(i)^2);
    
    % obtain the permutation
    icount = 1;
    permT = zeros(1, nbound);
    if i == 1 % the very first sample, accept all permutation
        while icount <= nbound
            per = randperm(cidx);
            pRank = rankMat(per,:);
            permT(icount) = sum((sum(wm.*(pRank((cidx-win(i)+1):cidx, :)-(cidx+1)/2), 1)).^2)/(sd(i)^2);
            icount = icount + 1;
        end
    else % generate samples conditional on previous acceptance
        permT = permuteEWMA(rankMat(1:cidx,:), nbound, climit, m0, minwin, maxwin, mu, sd, lambda);
    end

    % find the critical value of the test
    climit(i) = quantile(permT, 1-alpha);
    if Tval(i) > climit(i)
        runlength = i;
        break;
    end
%     toc;
end

varargout{1} = rawSample;
varargout{2} = rankMat;
varargout{2} = climit;
varargout{4} = Tval;
